//
// MOUSECONV.c
//
// for ATtiny2313
//
// FUSE
// L   = 0110_0100
// H   = 1101_1111
// EXT = 1111_1111
// (default)

#include <avr/interrupt.h>
#include <avr/io.h>

volatile uint8_t data_f = 0;
volatile uint8_t data_0 = 0;
volatile uint8_t data_1 = 0;
volatile uint8_t data_2 = 0;
volatile uint8_t data_3 = 0;
volatile uint8_t data_4 = 0;

volatile uint16_t	px = 0;
volatile uint16_t	py = 0;

volatile uint8_t px_u = 0;
volatile uint8_t px_l = 0;
volatile uint8_t py_u = 0;
volatile uint8_t py_l = 0;

volatile uint8_t doutcnt = 0;
volatile uint8_t doutx_l = 0;
volatile uint8_t douty_u = 0;
volatile uint8_t douty_l = 0;

volatile uint8_t readflag = 0;


ISR(INT1_vect) {

	if ( (PIND & 0x08) != 0) {
		if ( (doutcnt == 2) && ((TIFR & 0x02) == 0) ) { // 3rd
			doutcnt = 3;
			PORTB = douty_u;
		} else {			// 1st
			doutcnt = 1;
			PORTB = px_u;
			doutx_l = px_l;
			douty_u = py_u;
			douty_l = py_l;
			px = 0;
			py = 0;
			px_u = 0;
			px_l = 0;
			py_u = 0;
			py_l = 0;
		}
	} else {
		if (doutcnt == 1) {	// 2nd
			doutcnt = 2;
			PORTB = doutx_l;
			TCNT0 = 256-25;	// 200us timer
			TIFR |= 0x02;	// clear overflow flag
		} else {			// 4th
			doutcnt = 0;
			PORTB = douty_l;
		}
	}
}

int main(void)
{
	uint8_t		i;
	uint16_t	com_data;

	uint8_t		dx,dy;
	uint16_t	sumx,sumy;

	uint8_t		trg_c = 0;
	uint8_t		trg_c_old = 0;

	uint8_t		pxtmp,pytmp;
	uint8_t		speed = 0;


// clock setting
	CLKPR = 0x80;	// clock setting initialize
	CLKPR = 0x00;	// clock setting x1

// pin setting
	// D7   : (-) no assign
	// D6   : (-) open
	// D5   : (-) open
	// D4   : (b) PS2_DATA
	// D3   : (i) (INT1)P6_STROBE
	// D2   : (b) (INT0)PS2_CLK
	// D1   : (o) P6_TRGB
	// D0   : (o) P6_TRGA
	// B7   : (-) open(SCK)
	// B6   : (-) open(MISO)
	// B5   : (-) open(MOSI)
	// B4   : (-) open
	// B3   : (o) P6_DOUT3
	// B2   : (o) P6_DOUT2
	// B1   : (o) P6_DOUT1
	// B0   : (o) P6_DOUT0
	// A7   : (-) no assign
	// A6   : (-) no assign
	// A5   : (-) no assign
	// A4   : (-) no assign
	// A3   : (-) no assign
	// A2   : (-) open
	// A1   : (i) nRESET
	// A0   : (-) open

	DDRD  = 0x03;
	PORTD = 0xFF;
	DDRB  = 0x0F;
	PORTB = 0xFF;
	DDRA  = 0x00;
	PORTA = 0xFF;

// interrupt setting
	MCUCR = 0x06;	// INT1:rising or falling INT0:falling
	GIMSK = 0xC0;	// INT1,0 interrupt enable
	EIFR  = 0x00;	// interrupt flag reset

// mouse initialize sequense
	DDRD  = 0x07;	// PS2_CLK:out PS2_DATA:in
	PORTD = 0xFB;	// PS2_CLK:0   PS2_DATA:1

	// wait 800 us
	TCCR0B = 0x03;	// timer clock = sys-clk/64 (125KHz)
	TCNT0 = 256-100;
	TIFR |= 0x02;	// clear flag
	while ( (TIFR & 0x02) == 0) {
		;
	}

	sei();

	DDRD  = 0x17;	// PS2_CLK:out PS2_DATA:out
	PORTD = 0xEB;	// PS2_CLK:0   PS2_DATA:0

	PORTD = 0xEF;	// PS2_CLK:1   PS2_DATA:0
	DDRD  = 0x13;	// PS2_CLK:in  PS2_DATA:out


	com_data = 0x02F4;	// enable data reporting

	for (i = 0;i < 10;i ++) {
		data_f = 0;
		while (data_f == 0) {	// wait PS2_CLK
			;
		}
		if ((com_data & 0x0001) != 0) {
			PORTD |= 0x10;
		} else {
			PORTD &= 0xEF;
		}
		com_data = (com_data >> 1);
	}


	PORTD = 0xFF;	// PS2_CLK:1   PS2_DATA:1
	DDRD  = 0x03;	// PS2_CLK:in  PS2_DATA:in

	// read response data
	for (i = 0;i < 12;i ++) {
		data_f = 0;
		while (data_f == 0) {	// wait PS2_CLK
			;
		}
	}


// main loop
    for (;;) {

		for (i = 0;i < 33;i ++) {
			data_f = 0;
			while (data_f == 0) {	// wait PS2_CLK
				;
			}
		}

	//
	// data_0:7  6  5  4  3  2  1  0
	//        ED P  Y7 Y6 Y5 Y4 Y3 Y2
	//
	// data_1:7  6  5  4  3  2  1  0
	//        Y1 Y0 ST ED P  X7 X6 X5
	//
	// data_2:7  6  5  4  3  2  1  0
	//        X4 X3 X2 X1 X0 ST ED P 
	//
	// data_3:7  6  5  4  3  2  1  0
	//        YV XV YS XS 1  C  R  L 
	//
	// data_4:7  6  5  4  3  2  1  0
	//        ST
	//

	// error check ... no error check


	// mouse trigger output
		PORTD = ~(data_3 & 0x03);

		trg_c_old = trg_c;
		trg_c = (data_3 & 0x04);

		if (trg_c_old == 0 && trg_c != 0) {
			if (speed == 2) {
				speed = 0;
			} else {
				speed ++;
			}
		}

		dx = ((data_1 & 0x07) << 5) | ((data_2 & 0xF8) >> 3);
		dy = ((data_0 & 0x3F) << 2) | ((data_1 & 0xC0) >> 6);

	// overflow
		if ((data_3 & 0x40) != 0) {
			if ((data_3 & 0x10) != 0) {
				dx = 0x00;
			} else {
				dx = 0xFF;
			}
		}
		if ((data_3 & 0x80) != 0) {
			if ((data_3 & 0x20) != 0) {
				dy = 0x00;
			} else {
				dy = 0xFF;
			}
		}

	// expand pointer 9bit -> 16bit
		sumx = ((data_3 & 0x10) != 0) ? 0xFF00 : 0x0000;
		sumy = ((data_3 & 0x20) != 0) ? 0xFF00 : 0x0000;
		sumx |= dx;
		sumy |= dy;

	// invert X data
		sumx = ~sumx;
		sumx += 1;

	// update pointer
		cli();

		pxtmp = px;
		pytmp = py;

		px += sumx;
		py += sumy;

	// pointer overflow check
		if ( (pxtmp & 0x8000) != (px & 0x8000) ) {
			if        ( (pxtmp & 0xC000) == 0x4000 && (px & 0xC000) == 0x8000) {
				px = 0x7FFF;
			} else if ( (pxtmp & 0xC000) == 0x8000 && (px & 0xC000) == 0x4000) {
				px = 0x8000;
			}
		}

		if ( (pytmp & 0x8000) != (py & 0x8000) ) {
			if        ( (pytmp & 0xC000) == 0x4000 && (py & 0xC000) == 0x8000) {
				py = 0x7FFF;
			} else if ( (pytmp & 0xC000) == 0x8000 && (py & 0xC000) == 0x4000) {
				py = 0x8000;
			}
		}

	// pointer 16bit -> 8bit
		if ( (px & 0x8000) != 0) {				// px <  0
			if (speed == 0) {
				if ( (px & 0x7F80) != 0x7F80) {		// overflow
					pxtmp = 0x80;
				} else {
					pxtmp = (px & 0x00FF);
				}
			} else if (speed == 1) {
				if ( (px & 0x7F00) != 0x7F00) {		// overflow
					pxtmp = 0x80;
				} else {
					pxtmp = ( (px & 0x01FE) >> 1);
				}
			} else {
				if ( (px & 0x7E00) != 0x7E00) {		// overflow
					pxtmp = 0x80;
				} else {
					pxtmp = ( (px & 0x03FC) >> 2);
				}
			}
		} else {								// px >= 0
			if (speed == 0) {
				if ( (px & 0x7F80) != 0x0000) {		// overflow
					pxtmp = 0x7F;
				} else {
					pxtmp = (px & 0x00FF);
				}
			} else if (speed == 1) {
				if ( (px & 0x7F00) != 0x0000) {		// overflow
					pxtmp = 0x7F;
				} else {
					pxtmp = ( (px & 0x01FE) >> 1);
				}
			} else {
				if ( (px & 0x7E00) != 0x0000) {		// overflow
					pxtmp = 0x7F;
				} else {
					pxtmp = ( (px & 0x03FC) >> 2);
				}
			}
		}

		if ( (py & 0x8000) != 0) {				// py <  0
			if (speed == 0) {
				if ( (py & 0x7F80) != 0x7F80) {		// overflow
					pytmp = 0x80;
				} else {
					pytmp = (py & 0x00FF);
				}
			} else if (speed == 1) {
				if ( (py & 0x7F00) != 0x7F00) {		// overflow
					pytmp = 0x80;
				} else {
					pytmp = ( (py & 0x01FE) >> 1);
				}
			} else {
				if ( (py & 0x7E00) != 0x7E00) {		// overflow
					pytmp = 0x80;
				} else {
					pytmp = ( (py & 0x03FC) >> 2);
				}
			}
		} else {								// py >= 0
			if (speed == 0) {
				if ( (py & 0x7F80) != 0x0000) {		// overflow
					pytmp = 0x7F;
				} else {
					pytmp = (py & 0x00FF);
				}
			} else if (speed == 1) {
				if ( (py & 0x7F00) != 0x0000) {		// overflow
					pytmp = 0x7F;
				} else {
					pytmp = ( (py & 0x01FE) >> 1);
				}
			} else {
				if ( (py & 0x7E00) != 0x0000) {		// overflow
					pytmp = 0x7F;
				} else {
					pytmp = ( (py & 0x03FC) >> 2);
				}
			}
		}

		px_u = 0xF0 | ( (pxtmp & 0xF0) >> 4);
		px_l = 0xF0 |   (pxtmp & 0x0F);
		py_u = 0xF0 | ( (pytmp & 0xF0) >> 4);
		py_l = 0xF0 |   (pytmp & 0x0F);

		sei();


    }
}
